"""
pack and unpack primitive data types. See: https://docs.micropython.org/en/v1.19.1/library/struct.html

|see_cpython_module| :mod:`python:struct` https://docs.python.org/3/library/struct.html .

Supported size/byte order prefixes: ``@``, ``<``, ``>``, ``!``.

Supported format codes: ``b``, ``B``, ``h``, ``H``, ``i``, ``I``, ``l``,
``L``, ``q``, ``Q``, ``s``, ``P``, ``f``, ``d`` (the latter 2 depending
on the floating-point support).
"""
from typing import Tuple, Any

def pack_into(fmt, buffer, offset, v1, *args, **kwargs) -> Any:
    """
    Pack the values *v1*, *v2*, ... according to the format string *fmt*
    into a *buffer* starting at *offset*. *offset* may be negative to count
    from the end of *buffer*.
    """
    ...

def unpack(fmt, data) -> Tuple:
    """
    Unpack from the *data* according to the format string *fmt*.
    The return value is a tuple of the unpacked values.
    """
    ...

def unpack_from(fmt, data, offset=0, /) -> Tuple:
    """
    Unpack from the *data* starting at *offset* according to the format string
    *fmt*. *offset* may be negative to count from the end of *buffer*. The return
    value is a tuple of the unpacked values.
    """
    ...

def pack(fmt, v1, *args, **kwargs) -> bytes:
    """
    Pack the values *v1*, *v2*, ... according to the format string *fmt*.
    The return value is a bytes object encoding the values.
    """
    ...

def calcsize(fmt) -> int:
    """
    Return the number of bytes needed to store the given *fmt*.
    """
    ...



"""
random numbers. See: https://docs.micropython.org/en/v1.19.1/library/random.html

This module implements a pseudo-random number generator (PRNG).

|see_cpython_module| :mod:`python:random` https://docs.python.org/3/library/random.html . .

.. note::

   The following notation is used for intervals:

   - () are open interval brackets and do not include their endpoints.
     For example, (0, 1) means greater than 0 and less than 1.
     In set notation: (0, 1) = {x | 0 < x < 1}.

   - [] are closed interval brackets which include all their limit points.
     For example, [0, 1] means greater than or equal to 0 and less than
     or equal to 1.
     In set notation: [0, 1] = {x | 0 <= x <= 1}.

.. note::

   The :func:`randrange`, :func:`randint` and :func:`choice` functions are only
   available if the ``MICROPY_PY_URANDOM_EXTRA_FUNCS`` configuration option is
   enabled.

"""
from typing import Optional, Any

def seed(n=None, /) -> None:
    """
    Initialise the random number generator module with the seed *n* which should
    be an integer.  When no argument (or ``None``) is passed in it will (if
    supported by the port) initialise the PRNG with a true random number
    (usually a hardware generated random number).

    The ``None`` case only works if ``MICROPY_PY_URANDOM_SEED_INIT_FUNC`` is
    enabled by the port, otherwise it raises ``ValueError``.
    """
    ...

def getrandbits(n) -> int:
    """
    Return an integer with *n* random bits (0 <= n <= 32).
    """
    ...



from _typeshed import Incomplete

TAR_HEADER: Incomplete
DIRTYPE: str
REGTYPE: str

def roundup(val, align): ...

class FileSection:
    f: Incomplete
    content_len: Incomplete
    align: Incomplete
    def __init__(self, f, content_len, aligned_len) -> None: ...
    def read(self, sz: int = ...): ...
    def readinto(self, buf): ...
    def skip(self) -> None: ...

class TarInfo:
    def __str__(self): ...

class TarFile:
    f: Incomplete
    subf: Incomplete
    def __init__(self, name: Incomplete | None = ..., fileobj: Incomplete | None = ...) -> None: ...
    def next(self): ...
    def __iter__(self): ...
    def __next__(self): ...
    def extractfile(self, tarinfo): ...



from _typeshed import Incomplete
from neopixel import NeoPixel

class APA102(NeoPixel):
    ORDER: Incomplete
    clock_pin: Incomplete
    def __init__(self, clock_pin, data_pin, n, bpp: int = ...) -> None: ...
    def write(self) -> None: ...
